package top.yexuejc.demo.core;


import java.io.File;
import java.lang.reflect.Method;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import top.yexuejc.demo.annotation.GetMapping;
import top.yexuejc.demo.annotation.RestController;

/**
 * controller类提供者
 *
 * @author maxiaofeng
 * @date 2025/7/11 14:40
 */
public class ControllerSupplier {
    private final String scanPackage;
    private List<Class<?>> controllerClasses = new ArrayList<>();
    /** @GetMapping 路径映射 */
    private Map<String, CtrlBean> getMapping = new ConcurrentHashMap<>();

    private static volatile ControllerSupplier instance;


    private ControllerSupplier(String scanPackage) {
        this.scanPackage = scanPackage;
    }

    public static ControllerSupplier getInstance(String scanPackage) {
        if (instance == null) {
            synchronized (ControllerSupplier.class) {
                if (instance == null) {
                    instance = new ControllerSupplier(scanPackage);
                }
            }
        }
        return instance;
    }

    public static void builder(Class<?> applicationClass) {
        // 得到类的包名
        String packageName = applicationClass.getPackage().getName();
        ControllerSupplier controllerSupplier = ControllerSupplier.getInstance(packageName);
        controllerSupplier.load();
    }

    /**
     * 扫描包下的@RestController和@Controller注解的类
     */
    public void load() {
        try {
            // 获取包路径对应的 URL 资源
            String packagePath = scanPackage.replace(".", "/");
            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
            Enumeration<URL> resources = classLoader.getResources(packagePath);

            List<Class<?>> classes = new ArrayList<>();

            while (resources.hasMoreElements()) {
                File file = new File(resources.nextElement().getFile());
                if (file.isDirectory()) {
                    // 遍历目录下所有 .class 文件并加载类
                    List<Class<?>> foundClasses = getClassesFromDirectory(file, scanPackage);
                    classes.addAll(foundClasses);
                }
            }

            // 筛选出带有 @RestController 的类
            controllerClasses = classes.stream().filter(clazz -> clazz.isAnnotationPresent(RestController.class)).toList();

            controllerClasses.forEach(clazz -> {
                System.out.println("controller: " + clazz.getName());
                // 实例化类
                Object ctrl;
                try {
                    ctrl = clazz.getConstructor().newInstance();
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
                // 找到类下的@GetMapping 方法
                Arrays.stream(clazz.getMethods()).filter(method -> method.isAnnotationPresent(GetMapping.class)).forEach(method -> {
                    GetMapping getMappingAnnotation = method.getAnnotation(GetMapping.class);
                    String[] uris = getMappingAnnotation.value();
                    for (String uri : uris) {
                        System.out.println("uri: " + uri);
                        getMapping.put(uri, new CtrlBean(ctrl, method));
                    }
                });
            });

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static class CtrlBean {
        public final Object ctrl;
        public final Method method;

        public CtrlBean(Object ctrl, Method method) {
            this.ctrl = ctrl;
            this.method = method;
        }
    }

    /**
     * 从给定目录扫描并加载所有类
     */
    private List<Class<?>> getClassesFromDirectory(File directory, String packageName) throws ClassNotFoundException {
        List<Class<?>> classes = new ArrayList<>();
        if (!directory.exists()) {
            return classes;
        }

        File[] files = directory.listFiles();
        if (files == null) {
            return classes;
        }

        for (File file : files) {
            if (file.isDirectory()) {
                classes.addAll(getClassesFromDirectory(file, STR."\{packageName}.\{file.getName()}"));
            } else if (file.getName().endsWith(".class")) {
                String className = STR."\{packageName}.\{file.getName().substring(0, file.getName().length() - 6)}";
                Class<?> clazz = Class.forName(className);
                classes.add(clazz);
            }
        }

        return classes;
    }

    /**
     * 处理请求
     *
     * @param request
     * @return
     */
    public static Response invoke(Request request) {
        CtrlBean ctrlBean = ControllerSupplier.getInstance(null).getMapping.get(request.getPath());
        if (ctrlBean != null) {
            try {
                // 判断method是否有参数
                Object result;
                if (ctrlBean.method.getParameterCount() == 0) {
                    result = ctrlBean.method.invoke(ctrlBean.ctrl);
                } else {
                    result = ctrlBean.method.invoke(ctrlBean.ctrl, request.getParams());
                }
                return new Response(200, "OK", "text/html;charset=utf-8", result.toString().getBytes(StandardCharsets.UTF_8));
            } catch (Exception e) {
                e.printStackTrace();
                return new Response(500, "Internal Server Error", "text/html;charset=utf-8", "Internal Server Error".getBytes(StandardCharsets.UTF_8));
            }
        }
        return null;
    }

}

