package top.yexuejc.demo.core;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStore;
import java.util.Properties;
import java.util.logging.Logger;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;

import top.yexuejc.demo.AppConfig;

/**
 * web服务核心
 *
 * @author maxiaofeng
 * @date 2025/6/19 11:13
 */
public class WebServer {
    Logger logger = Logger.getLogger(WebServer.class.getName());
    /** 端口 */
    private int httpPort = 80;
    private int httpsPort = 443;
    /**
     * 证书格式：
     * 1. JKS
     * 2. PKCS12
     */
    private String httpsSslKeyStore = "PKCS12";
    private String httpsSslKeyStoreFile;
    private String httpsSslKeyStorePassword;
    /** 页面资源文件 */
    private String webRoot = "web";
    /** 静态资源文件 */
    private String staticRoot = "static";

    private ServerSocket serverSocket;
    private SSLServerSocket httpsServerSocket;
    private boolean isRunning;

    public WebServer() {
        Properties config = AppConfig.getProperties();
        String s = config.getProperty("http.port");
        httpPort = s == null ? 80 : Integer.parseInt(s);
        s = config.getProperty("https.port");
        httpsPort = s == null ? 443 : Integer.parseInt(s);
        s = config.getProperty("https.ssl.type");
        httpsSslKeyStore = s == null ? httpsSslKeyStore : s;
        s = config.getProperty("https.ssl.key-store");
        httpsSslKeyStoreFile = s == null ? httpsSslKeyStoreFile : s;
        s = config.getProperty("https.ssl.key-store-password");
        httpsSslKeyStorePassword = s == null ? httpsSslKeyStorePassword : s;
    }

    public WebServer(int httpPort, int httpsPort, String staticRoot, String webRoot) {
        this.httpPort = httpPort;
        this.httpsPort = httpsPort;
        this.staticRoot = staticRoot;
        this.webRoot = webRoot;
    }

    /**
     * 启动服务
     */
    public void start() {
        new Thread(this::startHttp).start();
        if (httpsSslKeyStoreFile != null && !httpsSslKeyStoreFile.isEmpty()) {
            new Thread(this::startHttps).start();
        }
    }

    public void startHttp() {
        try {
            serverSocket = new ServerSocket(httpPort);
            isRunning = true;
            logger.info("启动Web服务 : http://127.0.0.1:" + httpPort);

            // 阻塞式监听服务是否正常
            while (isRunning) {
                try {
                    Socket clientSocket = serverSocket.accept();
                    // 新建一个线程处理请求
                    new RequestHandler(clientSocket, staticRoot, webRoot).run();
                } catch (IOException e) {
                    if (isRunning) {
                        logger.severe("创建连接异常。");
                        e.printStackTrace();
                    }
                }
            }
        } catch (Throwable r) {
            logger.severe("启动Web服务异常。");
            r.printStackTrace();
        } finally {
            stop();
        }
    }

    public void startHttps() {
        try {
            // 1. 加载 PKCS12 密钥库
            KeyStore keyStore = KeyStore.getInstance(httpsSslKeyStore);
            InputStream inputStream;
            if (httpsSslKeyStoreFile.startsWith("classpath:")) {
                inputStream = this.getClass().getClassLoader().getResourceAsStream(httpsSslKeyStoreFile.substring(10));
            } else {
                inputStream = new FileInputStream(httpsSslKeyStoreFile);
            }
            keyStore.load(inputStream, httpsSslKeyStorePassword.toCharArray());
            // 2. 初始化 KeyManagerFactory
            KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
            keyManagerFactory.init(keyStore, httpsSslKeyStorePassword.toCharArray());

            // 3. 初始化 SSLContext（TLS 1.2 或 1.3）
            SSLContext sslContext = SSLContext.getInstance("TLSv1.3");  // 或 "TLS"
            sslContext.init(keyManagerFactory.getKeyManagers(), null, null);

            // 4. 创建 SSLServerSocket
            SSLServerSocketFactory sslServerSocketFactory = sslContext.getServerSocketFactory();
            SSLServerSocket httpsServerSocket = (SSLServerSocket) sslServerSocketFactory.createServerSocket(httpsPort);
            httpsServerSocket.setEnabledProtocols(new String[]{"TLSv1.2", "TLSv1.3"});

            logger.info("启动Web服务 : https://127.0.0.1:" + httpsPort);
            isRunning = true;
            while (isRunning) {
                // 新建一个线程处理请求
                new RequestHandler(httpsServerSocket.accept(), staticRoot, webRoot).run();
            }
        } catch (Exception e) {
            if (isRunning) {
                logger.severe("创建连接异常。");
                e.printStackTrace();
            }
        } finally {
            stop();
        }
    }

    /**
     * 停止服务
     */
    public void stop() {
        isRunning = false;
        try {
            if (serverSocket != null) {
                serverSocket.close();
            }
        } catch (IOException e) {
            logger.info("停止Web服务异常: " + e.getMessage());
        }
        try {
            if (httpsServerSocket != null) {
                httpsServerSocket.close();
            }
        } catch (IOException e) {
            logger.info("停止Web服务异常: " + e.getMessage());
        }
        System.out.println("Web服务停止.");
    }
}
